/*
	Art. Lebedev Studio (www.artlebedev.com)
	date:		2012
	author:		Grigoriy Odegov

	This file provides declarations of plugin interfaces for Popularis Configurator software
*/
#pragma once
#include <Windows.h>
#include <string>
#include <Shlwapi.h>
#include "IAppControl.h"
#pragma comment(lib, "Shlwapi.lib")

// Messages for the condition's HookTrigger method
#ifndef _MAINAPP
enum KeyMessages
{
	KM_KEYDOWN		= WM_USER+400+0x01,	
	KM_KEYUP		= WM_USER+400+0x02,
	/*
			Keyboard language was changed or a new keyboard layout was loaded.
			wParam - Handle to the window.
			lParam - Handle to a keyboard layout. 
			*/
	KM_SHELL_LANG	= WM_USER+400+0x03,
	/*
			The activation has changed to a different top-level, unowned window. 
			wParam - Handle to the activated window.
			lParam - The value is TRUE if the window is in full-screen mode, or FALSE otherwise. 
			*/
	KM_SHELL_WIN_ACTIVATED = WM_USER+400+0x04,
};
#endif
/*
	Plugin types
*/
enum MXPluginType
{
	mxp_condition,						// the layout condition
	mxp_render,							// display render plugin (draws images to displays)
	mxp_action,							// action plugin, for launching macros and scenarios
	mxp_enforce_dword = 0xFFFFFFFF
};

struct SDisplayInfo
{
	UINT	width;
	UINT	height;
	size_t	keycode;
};

/*
	Serializer class
*/
class ISerializer
{
public:
	virtual void SetData(const char*)=0;
};

//-----------------------------------------------------------------------------

class IPlugin;
/*
	Plugin parent type. 
	
	Used to store common data for all plugin instances

*/
class IPluginType
{
protected:
	IAppControl* AppControl;
	std::wstring ModuleName;
public:
	IPluginType(): AppControl(NULL)
	{
		wchar_t path[MAX_PATH];
		::GetModuleFileName(GetModuleHandle(NULL),path,MAX_PATH-1);
		ModuleName = ::PathFindFileName(path);
	};
	virtual ~IPluginType(){};

	virtual const wchar_t*		GetPluginName()=0;							// get a symbolic identifier of the plugin
	virtual MXPluginType		GetPluginType()=0;							// get a plugin type
	virtual const wchar_t*		GetPluginModuleName()						// get dll name of the plugin
	{
		return ModuleName.c_str();
	}
	virtual const wchar_t*		GetPluginCaption()=0;						// localized plugin name for displaying on GUI

	virtual void				FreeInstance(IPlugin* plg)=0;				// Free the plugin instance								//
	virtual void				Init(IAppControl* fb){AppControl = fb;}		// Init all instances with IAppControl interface
	virtual IAppControl*		GetAppControl(){return AppControl;}			// Get application controller
	virtual void				InitGUI(){};								// Create GUI
};

/*
	Plugin instance itself. Created by IPluginType interface
*/
class IPlugin
{
protected:
	IAppControl*	AppControl;
	IPlugin*		Pair;
	IPluginType*	PT;
	unsigned int	Type;
	WPARAM			Param;
	unsigned int	oled;
public:
	IPlugin(IPluginType* pt,unsigned int type, WPARAM param, unsigned int OLED)
		:Type(type),Param(param),PT(pt),oled(OLED), Pair(nullptr)
	{
		AppControl = pt->GetAppControl();
	}

	virtual ~IPlugin()
	{
		if (Pair && Pair->GetPair() == this)
			Pair->SetPair(nullptr);
	};	

	virtual HWND				GetPluginWindow()=0;									// Get a plugin GUI window

	IPluginType*				GetCreator(){return PT;}								// Get a parent IPluginType
	virtual MXPluginType		GetPluginType(){return PT->GetPluginType();};			// Get a plugin type
	virtual const wchar_t*		GetPluginCaption(){return PT->GetPluginCaption();};		// localized plugin name for displaying on GUI
	virtual unsigned int		GetOLED(){return oled;}

	virtual void RedrawGUI()=0;															// Calls UpdateWindow for plugin GUI

	virtual void Serialize(ISerializer* srlzr){};										// Serialize plugin data in provided ISerializer interface. Data must be formatted as UTF8 text
	virtual void Deserialize(const char* Data){};										// Deserialize plugin data, serialized with previous method
	virtual bool Animated(){return false;};												// If plugin needs a constant animation loop, this method should return true

	virtual IPlugin* GetPair(){return Pair;}
	virtual void SetPair(IPlugin* val)
	{
		Pair= val;
	}
	virtual void ClearPair()
	{
		if (Pair)
		{
			IPlugin* copy = Pair;
			Pair = nullptr;
			copy->ClearPair();
		}
	}
};

//-----------------------------------------------------------------------------

/*
	Condition plugin type

	Used to create condition plugin instances
*/
class IPCondition;
class IPConditionType : public IPluginType
{
public:
	virtual MXPluginType	GetPluginType() {return mxp_condition;}

	virtual IPCondition*	CreateInstance(unsigned int Type, WPARAM Param, unsigned int OLED)=0;			// Creates an instance of the IPCondition plugin
};


/*
	Condition instance

	If the developer wants to create his own condition plugin, he needs to inherit his class from IPCondition.
	There are the thinngs he has to implement:
	1. GUI window sized 240x20 pixels. The API used to implement it does not matter.
	2. Implement the IsActive method. This method is called by the configurator, when it decides to update the layouts.
	3. Implement SetBackgroundColor method. This method should set background color for GUI window, described in 1.
	4. if the plugin recieves some kind of event, desribed in IAppControl interface, the configurator will call HookTrigger method every time the event fires. There you can implement the handling

	If you want to create some kind of interoperability between the condition plugin and your own other application, you'll have to implement the interprocess message pump in a separate thread
	and call IAppControl::UpdateRequest(this) every time you want the Configurator to update layouts.
*/
class IPCondition : public IPlugin
{
public:
	IPCondition(IPluginType* pt, unsigned int Type, WPARAM Param, unsigned int OLED) : IPlugin(pt,Type,Param,OLED){};
	virtual bool				IsActive()=0;														// the inherited class 
	virtual void				HookTrigger(DWORD message, WPARAM wParam, LPARAM lParam){};			// this method is called if the event, the plugin subscribed, fires
	IPConditionType*			GetOwner(){return (IPConditionType*)PT;};							// get the parent IPluginType
	virtual void				SetBackgroundColor(COLORREF color)=0;								// Sets background color of the plugin GUI. Configurator calls it to set the background color in the layout treeview
	virtual HICON				GetIcon(){return nullptr;}											// Condition icon
};

//-----------------------------------------------------------------------------

/*
	Rendering plugin type
*/

class IPKeyRender;
class IPKeyRenderType : public IPluginType
{
public:
	virtual MXPluginType		GetPluginType(){return mxp_render;}

	virtual IPKeyRender*		CreateInstance(unsigned int Type, WPARAM Param, unsigned int OLED)=0;
	virtual void				ShowPluginWindow(IPKeyRender* plg)=0;
	virtual const wchar_t*		GetPair(){return L"";}
};

/*
	Rendering instance

	This plugin implements a render to the button display.

	When the configurator decides to redraw the display content, it calls the Draw method. This method provides HDC where you will make your GDI drawing calls. 
	Width and height parameters show the bitmap dimensions you are alowed to draw to. The key parameter provides the OLED display index.

	If the developer wants to create a dynamic plugin (with permanent redraw loop), it should implement the Animated method and make it return true:
		
		virtual bool Animated(){return true;}


*/
class IPKeyRender : public IPlugin
{
public:
	IPKeyRender(IPluginType* pt, unsigned int Type, WPARAM Param, unsigned int OLED) : IPlugin(pt,Type,Param,OLED){};
	virtual void				Draw(HDC dest, size_t key, unsigned int width, unsigned int height)=0;
	
	IPKeyRenderType*			GetOwner(){return (IPKeyRenderType*)PT;};
	virtual void				ImprotProperties(IPKeyRender* Src){};
};

//-----------------------------------------------------------------------------

class IPAction;
class IPActionType : public IPluginType
{
public:
	virtual MXPluginType		GetPluginType() {return mxp_action;}

	virtual void				ShowPluginWindow(IPAction* plg)=0;
	virtual IPAction*			CreateInstance(unsigned int Type, WPARAM Param, unsigned int OLED)=0;
};

/*
	The Action plugin.

	Plugin of this type is being assigned to the button and configurator calls it when user presses the corresponding button

	When this happens, the Action method is being called.

	The developer can do any work inside of this method.

*/

class IPAction : public IPlugin
{
public:
	IPAction(IPluginType* pt, unsigned int Type, WPARAM Param, unsigned int OLED) : IPlugin(pt,Type,Param,OLED){};
	virtual MXPluginType		GetPluginType(){return mxp_action;}

	virtual void				Action(DWORD message, WPARAM wParam, LPARAM lParam)=0;
	virtual bool				BlockInput(){return false;}
	virtual unsigned char		GetCurrentScanCode()
	{
		return AppControl->OLEDToScanCode(this->oled);
	}
	virtual void				SetCommandLine(const char* cmd){};

	IPActionType*				GetOwner(){return (IPActionType*)PT;};
	virtual void				Update(){};									// Called when key is activated
};

//--------------------------------------------------------------------------
